#!/bin/bash
#version 1.1
set -u

PRODUCT=POP
PRODUCT_LOWCASE=pop
upgrade_path=/etc/"$PRODUCT_LOWCASE"agent/upgrade
conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf
bak_conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf.upgradebak

logfile="/var/log/update.log"
target="version"
target_name=""
result_id=""

log()
{
    echo `date` "upgrade:" $@ >> $logfile
}

#write_upgrade_result_conf $1=success/failure
write_upgrade_result_conf()
{
    rm $bak_conf_file -f
    cp $conf_file $bak_conf_file -f
    if [ $? -eq 0 ];then
        sed -i "/upgraderesult/d" $bak_conf_file
        sed -i "/upgradeID/d" $bak_conf_file
        echo "upgraderesult=$1"  >> $bak_conf_file
        echo "upgradeID=${result_id}"  >> $bak_conf_file
        cp $bak_conf_file $conf_file -f
    fi
}
#upgrade_product_version $1=rpm_name
upgrade_product_version()
{
    local deb_name=$1
    log "uninstall $PRODUCT start"
    dpkg -r $PRODUCT >> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "uninstall success"
    else
        log "uninstall failed"
        return $ret
    fi
    log "install $PRODUCT $1 start"
    dpkg -i $deb_name>> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "install success"
    else
        log "install failed"
    fi
    return $ret
}

upgrade_product_version_rollback()
{
    log "UPGRADE version failed, try ROLLBACK"
    write_upgrade_result_conf "failure"
    deb=`dpkg -l | grep $PRODUCT`
    if [ -z $deb ];then
        for Line in $(ls -lt $upgrade_path | grep $PRODUCT | awk '{print $9}')
        do
            dpkg -i $upgrade_path/$Line >> $logfile
            deb=`dpkg -l| grep $PRODUCT`
            if [ -z $deb ];then
                log "install $Line failed , try again"
            else
                log "ROLLBACK to $Line success"
                log "sytemctl restart $PRODUCT"
                systemctl restart $PRODUCT
                exit
            fi
        done
        log "ROLLBACK failed , anyway exit"
        exit
    else
        log "sytemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit 
    fi
}

upgrade_version()
{
    log "----------------$(date)--------------------"
    log "Version upgrade, target=${target_name} result_id=${result_id}"
    if [ ! -f ${upgrade_path}/${target_name} ];then
        log "Could't find $target_name in directory ${upgrade_path}, upgrade ${result_id} failed"
        popagent upgradeFailed
        exit
    fi

    upgrade_product_version "${upgrade_path}/${target_name}"

    if [[ $? -eq 0 ]];then
        log "Upgrade ${target_name} upgradeId ${result_id} success"
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit
    else
        upgrade_product_version_rollback
    fi
}

manual_version_upgrade()
{
    log "----------------$(date)--------------------"
    log "Manual Version upgrade"
    if [[ ! -d $upgrade_path ]];then
        echo "$upgrade_path dir does not exist"
        log "$upgrade_path dir does not exist"
        exit
    fi
    
    target_name=`ls -lt $upgrade_path | grep -i $PRODUCT |grep deb| head -n1 | awk '{print $9}'`
    if [[ -z $target_name ]];then
        echo "Could not find any upgrade deb in $upgrade_path"
        log "Could not find any upgrade deb in $upgrade_path"
        exit
    fi

    upgrade_product_version "${upgrade_path}/${target_name}"
    if [[ $? -eq 0 ]]; then
        log "Manual Version upgrade success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
    else
        upgrade_product_version_rollback
    fi
}

help()
{
    echo "Usage: $0 --target=<version> --target_name=POP-3.5.1-20211112.x86_64.rpm --result_id=xxxx"
    echo "      --target/-t         : Target  version"
    echo "      --target_name/-tn   : Upgrade target name, e.g POP-3.5.1-20211112.x86_64.rpm"
    echo "      --result_id/-r      : upgrade Id"
}

for opt in ${@};
do
    str="="
    result=$(echo $opt | grep "$str")
    if [ -n $result ]; then
        k=${opt%=*}
        v=${opt#*=}
    else
        k=${opt}
    fi

    case ${k} in
        "--target"|"-t") target=${v};;
        "--target_name"|"-tn") target_name=${v};;
        "--result_id"|"-r") result_id=${v};;
        *) echo "${opt} is error";;
    esac
done

if [ $# -lt 3 ];then
    target="manual_version"
else
    [[ -z $target || -z $target_name || -z $result_id ]] && {
        help
        exit 1
    }
fi

if [ ${target} == "version" ]; then
    upgrade_version
elif [ ${target} == "manual_version" ]; then
    echo "manual_version"
    manual_version_upgrade
else
    log "----------------$(date)--------------------"
    log "Unsupported target ${target} result_id=${result_id}"
    popagent upgradeFailed
fi
