#!/usr/bin/sh
dir="/var/log/dcpe/"
# %10 of total disc space
dir_threshold_percent=10
# Disk usage, if this is exceeded, it is necessary to start checking the dir and cleaning up
disk_threshold_percent=60
disk_warning=false
dir_warning=false

log_file="/var/log/messages"
current_time=`date`
log_prefix="${current_time} <DcpeLogClean:>"

log()
{
	echo "${log_prefix}: $@" >> ${log_file}
}

df_result=$(df -k "$dir" 2>&1)
if [ $? -ne 0 ]; then
    log "Error running df command: $df_output"
    exit 1
fi

du_result=$(du -sk "$dir" 2>&1)
if [ $? -ne 0 ]; then
    log "Error running du command: $du_output"
    exit 1
fi

# Get dis usage and dir usage
disk_total=$(df -k "$dir" | awk 'NR==2 {print $2}')
dir_total=$(du -sk "$dir" | awk '{print $1}')
dir_usage=$((dir_total * 100 / disk_total))
disk_usage=$(df -h "$dir" | awk 'NR==2 {print $5}' | tr -d '%')
log "disk total:$disk_total    dir_total:$dir_total    dir_usage:$dir_usage    disk_usage:$disk_usage"

if ((disk_usage > disk_threshold_percent)); then
    disk_warning=true
    log "disk usage warning:$disk_usage%"
fi

if ((dir_usage > dir_threshold_percent)); then
    dir_warning=true
    log "dir usage warning:$dir_usage%"
fi

if $disk_warning && $dir_warning; then
    log "dir usage more than %$dir_threshold_percent, begin to cleanup..."
    # Sort the files by modification time 
    # Delete the oldest files until the space occupied is less than dir_threshold_percent
    while [ $dir_usage -gt $dir_threshold_percent ]; do
      oldest_file=$(ls -t "$dir" | tail -n 1)
      find "$dir" -maxdepth 1 -name "$dir$oldest_file" -execdir rm -rf {} \;
      rm -rf "$dir$oldest_file"
      log "rm file $dir$oldest_file"
      dir_total=$(du -sk "$dir" | awk '{print $1}')
      dir_usage=$((dir_total *100 / disk_total))
    done
    log "dir $dir space usage has returned to normal"
else
    log "dir $dir no need to clean up"
fi